// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:convert';
import 'dart:io';

import 'package:wyatt_cli_toolbox/src/prompt/keys.dart';
import 'package:wyatt_cli_toolbox/src/prompt/question.dart';
import 'package:wyatt_cli_toolbox/src/sequences/sequences.dart';
import 'package:wyatt_cli_toolbox/src/stylish/stylish.dart';

class PasswordQuestion extends Question<String> {
  PasswordQuestion(
    String question, {
    String? comment,
    required String identifier,
  }) : super(QuestionType.passwordQuestion, identifier, question, comment);

  String? answer;

  @override
  String ask() {
    if (stdin.hasTerminal) {
      stdin.echoMode = false;
      stdin.lineMode = false;
    }

    stdout.write(formatQuestion());
    String input = '';
    final List<int> bytes = <int>[];

    int lastB = stdin.readByteSync();
    while (lastB != Keys.enter) {
      stdout.write('*');
      bytes.add(lastB);
      lastB = stdin.readByteSync();
    }
    input = utf8.decode(bytes);
    stdout.write(formatQuestion());
    answer = List<String>.generate(input.length, (_) => '*')
        .reduce((String value, String element) => '$value$element');
    stdout.writeln(
      // ignore: lines_longer_than_80_chars
      '\r${formatQuestion()}${answer!.blue()}${Sequences.escClearLineFromCursorToEnd()}',
    );

    if (stdin.hasTerminal) {
      stdin.lineMode = true;
      stdin.echoMode = true;
    }
    return input;
  }
}
